/*
 *  TOPPERS Software
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 *
 *  Copyright (C) 2005-2008 by TAKAGI Nobuhisa  
 * 
 *  L쌠҂́Cȉ(1)`(4)̏𖞂ꍇɌC{\tgEF
 *  Ai{\tgEFAς̂܂ށDȉjgpEE
 *  ρEĔzziȉCpƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 *      ܂C{\tgEFÃ[U܂̓Gh[ÛȂ闝
 *      RɊÂCL쌠҂TOPPERSvWFNg
 *      Ɛӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̎gpړI
 *  ɑ΂K܂߂āCȂۏ؂sȂD܂C{\tgEF
 *  A̗pɂ蒼ړI܂͊ԐړIɐȂ鑹QɊւĂC
 *  ̐ӔC𕉂ȂD
 * 
 */

/*!
 *  \file   toppers/s_record.hpp
 *  \brief  g[SR[h߂̐錾`
 *
 *  ̃t@CŒ`NX
 *  \code
 *  class s_record;
 *  struct s_record::record;
 *  class s_record::format_error;
 *  class s_record::checksum_error;
 *  \endcode
 */
#ifndef TOPPERS_S_RECORD_HPP_
#define TOPPERS_S_RECORD_HPP_

#include "toppers/config.hpp"
#include <iosfwd>
#include <string>
#include <vector>
#include <utility>
#include <stdexcept>
#include <boost/cstdint.hpp>

namespace toppers
{

  /*!
   *  \class  s_record s_record.hpp "toppers/s_recored.hpp"
   *  \brief  g[SR[h߂̃NX
   *
   *  \sa s_record::record, s_record::format_error, s_record::checksum_error
   */
  class s_record
  {
  public:
    typedef std::pair< unsigned long, std::vector< unsigned char > > value_type;
    typedef std::vector< value_type >::size_type size_type;

    /*!
     *  \struct record s_record.hpp "toppers/s_record.hpp"
     *  \brief  SR[ĥPsR[hi[邽߂̍\
     */
    struct record
    {
      int type;
      int length;
      unsigned long address;
      std::vector< unsigned char > data;
      int checksum;
    };

    /*!
     *  \class  data_error s_record.hpp "toppers/s_record.hpp"
     *  \brief  SR[h̃f[^G[ONX
     */
    class data_error : public std::runtime_error
    {
    public:
      data_error() : std::runtime_error( "S-record data error" ) {}
    };
    /*!
     *  \class  format_error s_record.hpp "toppers/s_record.hpp"
     *  \brief  SR[h̏G[ONX
     */
    class format_error : public std::runtime_error
    {
    public:
      format_error() : std::runtime_error( "S-record format error" ) {}
    };
    /*!
     *  \class  checksum_error s_record.hpp "toppers/s_record.hpp"
     *  \brief  SR[h̃`FbNTG[ONX
     */
    class checksum_error : public std::runtime_error
    {
    public:
      checksum_error() : std::runtime_error( "S-record checksum error" ) {}
    };

    /*!
     *  \brief  ftHgRXgN^
     */
    s_record() : cache_( data_.end() ) {}
    /*!
     *  \brief  RXgN^
     *  \param  istr  ̓Xg[
     *
     *  s_record NX̐ƓɃf[^̃[hs܂B
     */
    explicit s_record( std::istream& istr ) { load( istr ); }
    /*!
     *  \brief  fXgN^
     */
    virtual ~s_record() {}

    void load( std::istream& istr );
    int operator[]( size_type address ) const;
    boost::uintmax_t get_value( std::size_t base, std::size_t size, bool little_endian ) const;
    unsigned long lower_bound() const;
    unsigned long upper_bound() const;
  protected:
    static record const read_record( std::string const& rec_buf );
    static int xdigit_to_int( int ch );
  private:
    std::vector< value_type > data_;
    mutable std::vector< value_type >::const_iterator cache_;
  };

}

#endif  // ! TOPPERS_S_RECORD_HPP_
